__all__ = [
    'actions',
    'add_command',
    'available_command_names',
    'commands',
    'core',
    'delete_command',
    'detailed_commands',
    'detailed_find_command',
    'display',
    'find_command',
    'table_view_commands',
    'table_view_find_command',
    'update_command',
]



from rich.prompt import Confirm
from .core import is_existing_store_name, switch_to_alt_store
from .messages import show_store_with_name_not_exists
from cmdcheatsheet.config.consts import CURR_STORE_LOCATION_CONF
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class SwitchToAltStore(CommandDetails):
    def __init__(self):
       super().__init__(
        ['--switch-to-alternative-store', '-stas'],
        "Switch to alternative store location.",
        [CommandArgument('store_name')])

    def handler(self, args):
        store_name = args[0]
        if is_existing_store_name(store_name):
            is_yes = Confirm.ask(
                f"If you switch the location, your config '{CURR_STORE_LOCATION_CONF}' " +
                "will be overridden, and you'll lose its value.\n" +
                "Do you want to proceed?"
            )
            if is_yes:
                switch_to_alt_store(store_name)
        else: 
            show_store_with_name_not_exists(store_name)



from rich import print
from rich.tree import Tree
from .consts import ALT_STORE_LOCATION_CONF, ALT_STORE_NAME_CONF, ALT_STORES_CONF
from .core import read_config
from .models import DisplayStrategy
from cmdcheatsheet.shared.display import BLUE, GREEN


class AltStoreDisplayStrategy(DisplayStrategy):

    def __init__(self):
        config = read_config()
        alt_stores = config.get(ALT_STORES_CONF)
        super().__init__(ALT_STORES_CONF, alt_stores)

    def display(self):
        if not self.config_value:
            print(f"[{BLUE}]{self.config_key}: [{GREEN}] <empty>")
            return
        tree = Tree(f"[{BLUE}]{self.config_key}:")
        for config_dict in self.config_value:
            tree.add(
                f"[{BLUE}]{config_dict[ALT_STORE_NAME_CONF]}:" +
                f"[{GREEN}]{config_dict[ALT_STORE_LOCATION_CONF]}"
            )
        print(tree)




from dataclasses import dataclass
from typing import Any


@dataclass
class DefaultConfig:
    key: str
    value: Any
    desc: str

class DefaultConfigurations:
    def __init__(self, configs=[]):
        self.configs = configs

    def __getitem__(self, config_key):
        return next((c for c in self.configs if c.key == config_key), None)

    def configs_as_dict(self):
        config_dict = dict()
        for config in self.configs:
            config_dict[config.key] = config.value
        return config_dict

    def configuration_keys(self):
        return [config.key for config in self.configs]

@dataclass
class DisplayStrategy:
    config_key: str
    config_value: Any

    def need_to_handle(self, config_key: str) -> bool:
        return self.config_key == config_key

    def display(self) -> None:
        raise NotImplementedError




__all__ = [
    'actions',
    'add_alt_store',
    'core',
    'delete_alt_store',
    'display',
    'display_applied_alt_store_name',
    'display_available_alt_stores',
    'messages',
    'models',
    'switch_to_alt_store',
    'update_alt_store',
]



from rich import print
from .alt_store_display_strategy import AltStoreDisplayStrategy
from .default_display_strategy import DefaultDisplayStrategy
from .core import read_config
from .consts import DEFAULT_CONFIG, CURR_STORE_LOCATION_CONF
from cmdcheatsheet.shared.display import BLUE, GREEN, display_error

DISPLAY_STRATEGIES = [
    DefaultDisplayStrategy(
        CURR_STORE_LOCATION_CONF,
        read_config().get(CURR_STORE_LOCATION_CONF)
    ),
    AltStoreDisplayStrategy()
]

def find_display_handler(config_key):
    return next(
        (ds for ds in DISPLAY_STRATEGIES if ds.need_to_handle(config_key)),
        None)

def display_configs(key=''): 
    if key:
        if key in DEFAULT_CONFIG.configuration_keys():
            display_single_config(key)
        else:
            display_error(f"No configuration found with the name {key}")
    else:
        config = read_config()
        for key in config:
            display_single_config(key)

def display_available_configs():
    for config in DEFAULT_CONFIG.configs:
        print(f"[{BLUE}]{config.key}[{GREEN}]: {config.desc}")

def display_single_config(config_key):
    handler = find_display_handler(config_key)
    handler.display()




from .commands import Commands
from .detailed_commands import DetailedCommands
from .table_view_commands import TableViewCommands
from .add_command import AddCommand
from .update_command import UpdateCommand
from .delete_command import DeleteCommand
from .find_command import FindCommand
from .detailed_find_command import DetailedFindCommand
from .table_view_find_command import TableViewFindCommand
from .available_command_names import AvailableCommandNames

command_actions = [
    Commands(),
    DetailedCommands(),
    TableViewCommands(),
    AddCommand(),
    UpdateCommand(),
    DeleteCommand(),
    FindCommand(),
    DetailedFindCommand(),
    TableViewFindCommand(),
    AvailableCommandNames()
]



from .core import is_existing_store_name, update_alt_store
from .messages import show_store_with_name_not_exists
from .models import AlternativeStore
from cmdcheatsheet.shared.messages import show_invalid_store_location_message
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails
from cmdcheatsheet.config.validators import is_valid_custom_commands_location


class UpdateAltStore(CommandDetails):
    def __init__(self):
       super().__init__(
        ['--update-alternative-store', '-uas'],
        "Update alternative commands store (JSON file).",
        [CommandArgument('store_name'), CommandArgument('store_location')])

    def handler(self, args):
        alt_store = AlternativeStore(args[0], args[1])
        if not is_existing_store_name(alt_store.name):
            show_store_with_name_not_exists(alt_store.name)            
        elif not is_valid_custom_commands_location(alt_store.location):
            show_invalid_store_location_message()
        else: 
            update_alt_store(alt_store)




from rich import print
from cmdcheatsheet.shared.display import BLUE, GREEN


def display_version(current_version):
    print(f"[{BLUE}] Version: [{GREEN}]{current_version}")

def display_help(command_name, description):
    print(f"[{BLUE}]  {command_name}[{GREEN}] - {description}")



from rich import print


BLUE = "turquoise2"
RED = "deep_pink2"
GREEN = "spring_green1"
YELLOW = "yellow"

def display_info(message):
    print(f"[{BLUE}]{message}")

def display_error(message):
    print(f"[{RED}]{message}")




from .display import display_commands_table_view
from cmdcheatsheet.shared.models import CommandDetails


class TableViewCommands(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--commands-table', '-ct'],
            'Display all commands using a table view.')

    def handler(self, _):
        display_commands_table_view()



from .display import display_commands_table_view
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class TableViewFindCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--find-table', '-ft'],
            'Search for a command and show it using a table view.',
            [CommandArgument('command')])

    def handler(self, args):
        display_commands_table_view(command=args[0])




from rich import print
from .models import DisplayStrategy
from cmdcheatsheet.shared.display import BLUE, GREEN


class DefaultDisplayStrategy(DisplayStrategy):

    def __init__(self, config_key, config_value):
        super().__init__(config_key, config_value)

    def display(self):
        print(f"[{BLUE}]{self.config_key}[{GREEN}]: {self.config_value}")

